<?php


namespace mcg76\world\map;

use pocketmine\block\Block;
use pocketmine\block\CoalOre;
use pocketmine\block\DiamondOre;
use pocketmine\block\Dirt;
use pocketmine\block\GoldOre;
use pocketmine\block\Gravel;
use pocketmine\block\IronOre;
use pocketmine\block\LapisOre;
use pocketmine\block\RedstoneOre;
use pocketmine\level\generator\noise\Simplex;
use pocketmine\level\generator\object\OreType;
use pocketmine\level\generator\populator\Ore;
use pocketmine\level\generator\populator\Populator;
use pocketmine\level\generator\populator\TallGrass;
use pocketmine\level\generator\populator\Tree;
use pocketmine\math\Vector3 as Vector3;
use pocketmine\utils\Random;
use pocketmine\level\ChunkManager;
use pocketmine\level\generator\Generator;
use pocketmine\block\Sand;

/**
 * MCG76 Desert
 *
 * Copyright (C) 2014 minecraftgenius76
 * YouTube Channel: http://www.youtube.com/user/minecraftgenius76
 *
 @author minecraftgenius76@gmail.com
 *
 */

class Desert extends Generator {
	
	/**
	 * @var Populator[]
	 */
	private $populators = [ ];
	/**
	 * @var ChunkManager
	 */
	private $level;
	/**
	 * @var Random
	 */
	private $random;
	private $worldHeight = 38;
	private $waterHeight = 32;
	private $noiseHills;
	private $noisePatches;
	private $noisePatchesSmall;
	private $noiseBase;
	public function __construct(array $options = []) {
	}
	public function getName() : string{
		return "desert";
	}
	public function getSettings() : array{
		return [];
	}
	public function init(ChunkManager $level, Random $random) {
		$this->level = $level;
		$this->random = $random;
		$this->random->setSeed ( $this->level->getSeed () );
		$this->noiseHills = new Simplex ( $this->random, 3, 0.11, 12 );
		$this->noisePatches = new Simplex ( $this->random, 2, 0.03, 16 );
		$this->noisePatchesSmall = new Simplex ( $this->random, 2, 0.5, 4 );
		$this->noiseBase = new Simplex ( $this->random, 16, 0.7, 16 );
		
		$ores = new Ore ();
		$ores->setOreTypes ( [ 
				new OreType(new CoalOre (), 20, 16, 0, 128 ),
				new OreType(new IronOre (), 20, 8, 0, 64 ),
				new OreType(new RedstoneOre (), 8, 7, 0, 16 ),
				new OreType(new LapisOre (), 1, 6, 0, 32 ),
				new OreType(new GoldOre (), 2, 8, 0, 32 ),
				new OreType(new DiamondOre (), 1, 7, 0, 16 ),
				new OreType(new Dirt (), 20, 32, 0, 128 ),
				new OreType(new Sand (), 20, 32, 0, 128 ),
				new OreType(new Gravel (), 10, 16, 0, 128 ) 
		] );
		$this->populators [] = $ores;
	}
	public function generateChunk(int $chunkX, int $chunkZ) {
		$this->random->setSeed ( 0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed () );
		$hills = [ ];
		$patches = [ ];
		$patchesSmall = [ ];
		$base = [ ];
		for($z = 0; $z < 16; ++ $z) {
			for($x = 0; $x < 16; ++ $x) {
				$i = ($z << 4) + $x;
				$hills [$i] = $this->noiseHills->noise2D ( $x + ($chunkX << 4), $z + ($chunkZ << 4), true );
				$patches [$i] = $this->noisePatches->noise2D ( $x + ($chunkX << 4), $z + ($chunkZ << 4), true );
				$patchesSmall [$i] = $this->noisePatchesSmall->noise2D ( $x + ($chunkX << 4), $z + ($chunkZ << 4), true );
				$base [$i] = $this->noiseBase->noise2D ( $x + ($chunkX << 4), $z + ($chunkZ << 4), true );
				
				if ($base [$i] < 0) {
					$base [$i] *= 0.5;
				}
			}
		}
		
		$chunk = $this->level->getChunk ( $chunkX, $chunkZ );
		
		for($chunkY = 0; $chunkY < 8; ++ $chunkY) {
			$startY = $chunkY << 4;
			$endY = $startY + 16;
			for($z = 0; $z < 16; ++ $z) {
				for($x = 0; $x < 16; ++ $x) {
					$i = ($z << 4) + $x;
					$height = $this->worldHeight + $hills [$i] * 14 + $base [$i] * 7;
					$height = ( int ) $height;
					
					for($y = $startY; $y < $endY; ++ $y) {
						$diff = $height - $y;
						if ($y <= 4 and ($y === 0 or $this->random->nextFloat () < 0.75)) {
							$chunk->setBlockId ( $x, $y, $z, Block::BEDROCK );
						} elseif ($diff > 2) {
							$chunk->setBlockId ( $x, $y, $z, Block::SANDSTONE );
						} elseif ($diff > 0) {
							if ($patches [$i] > 0.7) {
								$chunk->setBlockId ( $x, $y, $z, Block::COBBLESTONE );
							} elseif ($patches [$i] < - 0.8) {
								$chunk->setBlockId ( $x, $y, $z, Block::END_STONE );
							} else {
								$chunk->setBlockId ( $x, $y, $z, Block::SANDSTONE );
							}
						} elseif ($y <= $this->waterHeight) {
							if (($this->waterHeight - $y) <= 1 and $diff === 0) {
								$chunk->setBlockId ( $x, $y, $z, Block::SAND );
							} elseif ($diff === 0) {
								if ($patchesSmall [$i] > 0.6) {
									$chunk->setBlockId ( $x, $y, $z, Block::END_STONE );
								} elseif ($patchesSmall [$i] < - 0.45) {
									$chunk->setBlockId ( $x, $y, $z, Block::SAND );
								} else {
									$chunk->setBlockId ( $x, $y, $z, Block::DIRT );
								}
							} else {
								$chunk->setBlockId ( $x, $y, $z, Block::STILL_WATER );
							}
						} elseif ($diff === 0) {
							if ($patches [$i] > 0.7) {
								$chunk->setBlockId ( $x, $y, $z, Block::SANDSTONE );
							} elseif ($patches [$i] < - 0.8) {
								$chunk->setBlockId ( $x, $y, $z, Block::GRAVEL );
							} elseif ($patches [$i] > - 0.9 && $patches [$i] < - 1.0) {
								$chunk->setBlockId ( $x, $y, $z, Block::DEAD_BUSH );
							} else {
								$i = rand ( 0, 30 );
								if ($i == 0) {
									$p = rand ( 0, 8 );
									if ($p == 0) {
										$chunk->setBlockId ( $x, $y, $z, Block::SAND );
										$chunk->setBlockId ( $x, $y + 1, $z, Block::DEAD_BUSH );
									}
								} elseif ($i == 1) {
									$p = rand ( 0, 8 );
									if ($p == 0) {
										$chunk->setBlockId ( $x, $y, $z, Block::SAND );
										$chunk->setBlockId ( $x, $y + 1, $z, Block::CACTUS );
									}
								} elseif ($i == 2) {
									$p = rand ( 0, 18 );
									if ($p == 0) {
										$chunk->setBlockId ( $x, $y, $z, Block::GLOWSTONE );
										$chunk->setBlockId ( $x, $y+1, $z, Block::CHEST );
									}
								} else {
									$chunk->setBlockId ( $x, $y, $z, Block::SAND );
								}
							}
						}
					}
				}
			}
		}
	}
	public function populateChunk(int $chunkX, int $chunkZ) {
		$this->random->setSeed ( 0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed () );
		foreach ( $this->populators as $populator ) {
			$this->random->setSeed ( 0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed () );
			$populator->populate ( $this->level, $chunkX, $chunkZ, $this->random );
		}
	}
	public function getSpawn() : Vector3{
		return $this->level->getSafeSpawn ( new Vector3 ( 127.5, 128, 127.5 ) );
	}
}