<?php

namespace mcg76\world\map;

use pocketmine\block\Block;
use pocketmine\block\CoalOre;
use pocketmine\block\DiamondOre;
use pocketmine\block\Dirt;
use pocketmine\block\GoldOre;
use pocketmine\block\Gravel;
use pocketmine\block\IronOre;
use pocketmine\block\LapisOre;
use pocketmine\block\RedstoneOre;
use pocketmine\level\generator\noise\Simplex;
use pocketmine\level\generator\object\OreType;
use pocketmine\level\generator\populator\Ore;
use pocketmine\level\generator\populator\Populator;
use pocketmine\level\generator\populator\TallGrass;
use pocketmine\level\generator\populator\Tree;
use pocketmine\math\Vector3 as Vector3;
use pocketmine\utils\Random;
use pocketmine\level\ChunkManager;
use pocketmine\level\generator\Generator;
use mcg76\world\populator\HGTree;
use pocketmine\block\EndPortalFrame;
use pocketmine\block\EndStone;
use pocketmine\block\NetherBrick;
use pocketmine\block\Obsidian;

class Volcano extends Generator{

	/** @var Populator[] */
	private $populators = [];
	/** @var ChunkManager */
	private $level;
	/** @var Random */
	private $random;
	private $worldHeight = 74;
	private $waterHeight = 70;
	/** @var Simplex */
	private $noiseHills;
	/** @var Simplex */
	private $noiseBase;

	public function __construct(array $options = []){

	}

	public function getName() : string{
		return "volcano";
	}

	public function getSettings(): array{
		return [];
	}

	public function init(ChunkManager $level, Random $random){
		$this->level = $level;
		$this->random = $random;
		$this->random->setSeed($this->level->getSeed());
		$this->noiseHills = new Simplex($this->random, 3, 0.1, 12);
		$this->noiseBase = new Simplex($this->random, 16, 0.6, 16);


		$ores = new Ore();
		$ores->setOreTypes([
			new OreType(new EndStone(), 20, 16, 0, 128),
			new OreType(new NetherBrick(), 20, 32, 0, 128),
			new OreType(new Obsidian(), 10, 16, 0, 128),
		]);
		$this->populators[] = $ores;

// 		$trees = new HGTree ();
// 		//$trees = new Tree();
// 		$trees->setBaseAmount(2);
// 		$trees->setRandomAmount(1);
// 		$this->populators[] = $trees;

// 		$tallGrass = new TallGrass();
// 		$tallGrass->setBaseAmount(5);
// 		$tallGrass->setRandomAmount(3);
// 		$this->populators[] = $tallGrass;
	}

public function generateChunk(int $chunkX, int $chunkZ){
		$this->random->setSeed(0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed());
		$hills = [];
		$base = [];
		for($z = 0; $z < 16; ++$z){
			for($x = 0; $x < 16; ++$x){
				$i = ($z << 4) + $x;
				$hills[$i] = $this->noiseHills->noise2D($x + ($chunkX << 4), $z + ($chunkZ << 4), true);
				$base[$i] = $this->noiseBase->noise2D($x + ($chunkX << 4), $z + ($chunkZ << 4), true);

				if($base[$i] < 0){
					$base[$i] *= 0.5;
				}
			}
		}

		$chunk = $this->level->getChunk($chunkX, $chunkZ);

		for($z = 0; $z < 16; ++$z){
			for($x = 0; $x < 16; ++$x){
				$i = ($z << 4) + $x;
				$height = $this->worldHeight + $hills[$i] * 14 + $base[$i] * 7;
				$height = (int) $height;

				for($y = 0; $y < 128; ++$y){
					$diff = $height - $y;
					if($y <= 4 and ($y === 0 or $this->random->nextFloat() < 0.75)){
						$chunk->setBlockId($x, $y, $z, Block::BEDROCK);
					}elseif($diff > 2){
						$chunk->setBlockId($x, $y, $z, Block::END_STONE);
					}elseif($diff > 0){
						$chunk->setBlockId($x, $y, $z, Block::OBSIDIAN);
					}elseif($y <= $this->waterHeight){
						if(($this->waterHeight - $y) <= 1 and $diff === 0){
							$chunk->setBlockId($x, $y, $z, Block::NETHER_BRICK);
						}elseif($diff === 0){
							$chunk->setBlockId($x, $y, $z, Block::END_STONE);
						}else{
							$chunk->setBlockId($x, $y, $z, Block::LAVA);
						}
					}elseif($diff === 0){
						$chunk->setBlockId($x, $y, $z, Block::OBSIDIAN);
						
						$k = rand ( 0, 30 );
						if ($k == 1) {
							$chunk->setBlockId ($x, $y, $z, Block::LAVA);
						} elseif ($k == 6) {
							$chunk->setBlockId ($x, $y, $z, Block::NETHER_BRICK);
						} elseif ($k == 7) {
								$chunk->setBlockId ($x, $y, $z, Block::OBSIDIAN);
						} else {
							$chunk->setBlockId ($x, $y, $z, Block::OBSIDIAN);
						}
					}
				}

			}
		}

	}

	public function populateChunk(int $chunkX, int $chunkZ){
		$this->random->setSeed(0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed());
		foreach($this->populators as $populator){
			$this->random->setSeed(0xdeadbeef ^ ($chunkX << 8) ^ $chunkZ ^ $this->level->getSeed());
			$populator->populate($this->level, $chunkX, $chunkZ, $this->random);
		}
	}

	public function getSpawn() : Vector3{
		return $this->level->getSafeSpawn(new Vector3(127.5, 128, 127.5));
	}

}